/*
 Standalone Temperature/Voltage Logger for ATtiny85
 Version: 1.1
 Author: Alex from insideGadgets (www.insidegadgets.com)
 Created: 10/03/2012
 Last Modified: 5/08/2012

 */

#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#ifndef boolean
	typedef uint8_t boolean;
#endif
#ifndef bool
	typedef uint8_t bool;
#endif 
#ifndef byte
	typedef uint8_t byte;
#endif

#ifndef NULL
#define NULL ((void *)0)
#endif

#define LOW 0
#define HIGH 1
#define false 0
#define true 1
#define SKIPLEDOFFDELAY 1

#define T16MS 0
#define T32MS 1
#define T64MS 2
#define T128MS 3
#define T250MS 4
#define T500MS 5
#define T1S 6
#define T2S 7
#define T4S 8
#define T8S 9

#define eeprom1Kbit 127
#define eeprom2Kbit 255
#define eeprom4Kbit 511
#define eeprom8Kbit 1023
#define eeprom16Kbit 2047
#define eeprom32Kbit 4095
#define eeprom64Kbit 8191
#define eeprom128Kbit 16383
#define eeprom256Kbit 32767
#define eeprom512Kbit 65535

#define eepromA0 (1<<1) // 00000010
#define eepromA1 (1<<2) // 00000100
#define eepromA2 (1<<3) // 00001000

#define NOREPEATSTART 0
#define REPEATSTART 1

#define WAITINPUT 0
#define STARTLOGGING 1
#define CONFIGDELAY 2
#define CONFIGFUNCTION 3

#define I2C_DELAY_USEC 5
#define I2C_READ 1
#define I2C_WRITE 0

// Maps the button presses to the delay time
int delayTimemap[12] = {
	7, // 28 seconds
	15, // 60 seconds
	(5 * 15), // 5 minutes
	(10 * 15), // 10 minutes
	(15 * 15),
	(30 * 15),
	(60 * 15), // 1 hour
	(2 * 60 * 15), // 2 hours
	(4 * 60 * 15),
	(8 * 60 * 15),
	(12 * 60 * 15),
	(24 * 60 * 15), // 24 hours
};

// Maps the EEPROM last writeable location
uint16_t eepromMap[10] = {
	eeprom1Kbit,
	eeprom2Kbit,
	eeprom4Kbit,
	eeprom8Kbit,
	eeprom16Kbit,
	eeprom32Kbit,
	eeprom64Kbit,
	eeprom128Kbit,
	eeprom256Kbit,
	eeprom512Kbit,
};

// Initialize SCL/SDA pin and set the bus high
void SoftI2cMasterInit(void) {
	cbi(GIMSK,PCIE); // Disable pin change interrupt as SoftI2C will trigger it
	DDRB |= (1<<TWI_SDA_PIN);
	PORTB |= (1<<TWI_SDA_PIN);
	PORTB |= (1<<TWI_SCL_PIN);
}

// De-initialize SCL/SDA pins and set the bus low
void SoftI2cMasterDeInit(void) {
	PORTB &= ~(1<<TWI_SDA_PIN);
	DDRB &= ~(1<<TWI_SDA_PIN);
	PORTB &= ~(1<<TWI_SCL_PIN);
	sbi(GIMSK,PCIE); // Re-enable pin change interrupt
}

// Read a byte from I2C and send Ack if more reads follow else Nak to terminate read
uint8_t SoftI2cMasterRead(uint8_t last) {
	uint8_t b = 0;
	// Make sure pull-up enabled
	PORTB |= (1<<TWI_SDA_PIN);
	DDRB &= ~(1<<TWI_SDA_PIN);
	// Read byte
	for (uint8_t i = 0; i < 8; i++) {
		// Don't change this loop unless you verify the change with a scope
		b <<= 1;
		_delay_us(I2C_DELAY_USEC);
		PORTB |= (1<<TWI_SCL_PIN);
		_delay_us(I2C_DELAY_USEC); // Extra delay (needed when 16.5MHz)
		if (bit_is_set(PINB, TWI_SDA_PIN)) b |= 1;
		PORTB &= ~(1<<TWI_SCL_PIN);
	}
	// Send Ack or Nak
	DDRB |= (1<<TWI_SDA_PIN);
	if (last) { 
		PORTB |= (1<<TWI_SDA_PIN); 
	}
	else { 
		PORTB &= ~(1<<TWI_SDA_PIN);
	}
	_delay_us(I2C_DELAY_USEC); // Extra delay (needed when 16.5MHz)
	PORTB |= (1<<TWI_SCL_PIN);
	_delay_us(I2C_DELAY_USEC);
	PORTB &= ~(1<<TWI_SCL_PIN);
	PORTB &= ~(1<<TWI_SDA_PIN);
	_delay_us(I2C_DELAY_USEC); // Extra delay (needed when 16.5MHz)
	return b;
}

// Write a byte to I2C
bool SoftI2cMasterWrite(uint8_t data) {
	// Write byte
	for (uint8_t m = 0x80; m != 0; m >>= 1) {
		// Don't change this loop unless you verify the change with a scope
		if (m & data) { 
			PORTB |= (1<<TWI_SDA_PIN); 
		}
		else { 
			PORTB &= ~(1<<TWI_SDA_PIN); 
		}
		_delay_us(I2C_DELAY_USEC); // Extra delay (needed when 16.5MHz)
		PORTB |= (1<<TWI_SCL_PIN);
		_delay_us(I2C_DELAY_USEC);
		PORTB &= ~(1<<TWI_SCL_PIN);
	}
	_delay_us(I2C_DELAY_USEC); // Extra delay (needed when 16.5MHz)
	// get Ack or Nak
	DDRB &= ~(1<<TWI_SDA_PIN);
	_delay_us(1); // Extra delay (needed when 16.5MHz)
	// Enable pullup
	PORTB |= (1<<TWI_SDA_PIN);
	PORTB |= (1<<TWI_SCL_PIN);
	_delay_us(I2C_DELAY_USEC); // Extra delay (needed when 16.5MHz)
	uint8_t rtn = bit_is_set(PINB, TWI_SDA_PIN);
	PORTB &= ~(1<<TWI_SCL_PIN);
	PORTB &= ~(1<<TWI_SDA_PIN);
	DDRB |= (1<<TWI_SDA_PIN);
	_delay_us(I2C_DELAY_USEC); // Extra delay (needed when 16.5MHz)
	return rtn == 0;
}

// Issue a start condition
bool SoftI2cMasterStart(uint8_t addressRW) {
	PORTB &= ~(1<<TWI_SDA_PIN);
	_delay_us(I2C_DELAY_USEC);
	PORTB &= ~(1<<TWI_SCL_PIN);
	_delay_us(I2C_DELAY_USEC); // Extra delay (needed when 16.5MHz)
	return SoftI2cMasterWrite(addressRW);
}

// Issue a restart condition
bool SoftI2cMasterRestart(uint8_t addressRW) {
	PORTB |= (1<<TWI_SDA_PIN);
	PORTB |= (1<<TWI_SCL_PIN);
	_delay_us(I2C_DELAY_USEC);
	return SoftI2cMasterStart(addressRW);
}

// Issue a stop condition
void SoftI2cMasterStop(void) {
	PORTB &= ~(1<<TWI_SDA_PIN);
	_delay_us(I2C_DELAY_USEC);
	PORTB |= (1<<TWI_SCL_PIN);
	_delay_us(I2C_DELAY_USEC);
	PORTB |= (1<<TWI_SDA_PIN);
	_delay_us(I2C_DELAY_USEC);
}

// Issue the device start and the address to write
// 8 bit for 1-2Kbit, 8bit with A2-A0 as high bits for 4-16Kbit, 16bit for higher
bool SoftI2cMasterAddress(uint8_t deviceAddr, uint16_t addressRW, bool repeatedStart) {
	if (eepromMap[eepromMemsize] <= eeprom2Kbit) { // 1-2Kbit EEPROM
		if (!SoftI2cMasterStart((deviceAddr<<1) | I2C_WRITE)) return false;
		if (!SoftI2cMasterWrite(addressRW)) return false;
		if (repeatedStart) { if (!SoftI2cMasterRestart((deviceAddr<<1) | I2C_READ)) return false; }
	}
	else if (eepromMap[eepromMemsize] >= eeprom4Kbit && eepromMap[eepromMemsize] <= eeprom16Kbit) { // 4-16Kbit EEPROM
		if (addressRW <= eeprom2Kbit) { // No A2-A0 bits need to be set
			if (!SoftI2cMasterStart((deviceAddr<<1) | I2C_WRITE)) return false;
			if (!SoftI2cMasterWrite(addressRW)) return false;
			if (repeatedStart) { if (!SoftI2cMasterRestart((deviceAddr<<1) | I2C_READ)) return false; }
		}
		if (addressRW > eeprom2Kbit && addressRW <= eeprom4Kbit) { // Set A0 bit
			if (!SoftI2cMasterStart(((deviceAddr<<1) | I2C_WRITE) | eepromA0)) return false;
			if (!SoftI2cMasterWrite((addressRW & 0xFF))) return false;
			if (repeatedStart) { if (!SoftI2cMasterRestart(((deviceAddr<<1) | I2C_READ) | eepromA0)) return false; }
		}
		if (addressRW > eeprom4Kbit && addressRW <= eeprom8Kbit) { // Set A1 bit
			if (!SoftI2cMasterStart(((deviceAddr<<1) | I2C_WRITE) | eepromA1)) return false;
			if (!SoftI2cMasterWrite((addressRW & 0xFF))) return false;
			if (repeatedStart) { if (!SoftI2cMasterRestart(((deviceAddr<<1) | I2C_READ) | eepromA1)) return false; }
		}
		if (addressRW > eeprom8Kbit && addressRW <= eeprom16Kbit) { // Set A2 bit
			if (!SoftI2cMasterStart(((deviceAddr<<1) | I2C_WRITE) | eepromA2)) return false;
			if (!SoftI2cMasterWrite((addressRW & 0xFF))) return false;
			if (repeatedStart) { if (!SoftI2cMasterRestart(((deviceAddr<<1) | I2C_READ) | eepromA2)) return false; }
		}
	}
	else { // More than 16Kbit EEPROM
		if (!SoftI2cMasterStart((deviceAddr<<1) | I2C_WRITE)) return false;
		if (!SoftI2cMasterWrite((addressRW >> 8))) return false; // MSB
		if (!SoftI2cMasterWrite((addressRW & 0xFF))) return false; // LSB
		if (repeatedStart) { if (!SoftI2cMasterRestart((deviceAddr<<1) | I2C_READ)) return false; }
	}
	
	return true;
}

// Read 1 byte from the EEPROM device and return it
uint8_t soft_i2c_eeprom_read_byte(uint8_t deviceAddr, uint16_t readAddress) {
	uint8_t byteRead = 0;
	
	// Issue start condition, send device address, issue repeated start
	SoftI2cMasterAddress(deviceAddr, readAddress, REPEATSTART);
	
	// Read the byte
	byteRead = SoftI2cMasterRead(1);
	
	// Issue a stop condition
	SoftI2cMasterStop();
	
	return byteRead;
}

// Write 1 byte to the EEPROM
bool soft_i2c_eeprom_write_byte(uint8_t deviceAddr, uint16_t writeAddress, byte writeByte) {
	
	// Issue start condition and send device address
	SoftI2cMasterAddress(deviceAddr, writeAddress, NOREPEATSTART);
	
	// Write the byte
	if (!SoftI2cMasterWrite(writeByte)) return false;
	
	// Issue a stop condition
	SoftI2cMasterStop();
	
	// Delay 10ms for the write to complete, depends on the EEPROM you use
	// We divide this by 16.5MHz as we are at 1MHz when doing this delay
	_delay_us(650);
	
	return true;
}

// Read 1 byte from the address without caring about the EEPROM size
// This function should not be used by anything other than eepromSizetest
uint8_t soft_i2c_eeprom_read_minimal(uint8_t deviceAddr, uint16_t readAddress) {
	uint8_t byteRead = 0;
	
	if (readAddress <= eeprom2Kbit) { // 1-2Kbit EEPROM
		if (!SoftI2cMasterStart((deviceAddr<<1) | I2C_WRITE)) return false;
		if (!SoftI2cMasterWrite(readAddress)) return false;
		if (!SoftI2cMasterRestart((deviceAddr<<1) | I2C_READ)) return false;
	}
	else if (readAddress > eeprom2Kbit && readAddress <= eeprom4Kbit) { // Set A0 bit
		if (!SoftI2cMasterStart(((deviceAddr<<1) | I2C_WRITE) | eepromA0)) return false;
		if (!SoftI2cMasterWrite((readAddress & 0xFF))) return false;
		if (!SoftI2cMasterRestart(((deviceAddr<<1) | I2C_READ) | eepromA0)) return false;
	}
	else if (readAddress > eeprom4Kbit && readAddress <= eeprom8Kbit) { // Set A1 bit
		if (!SoftI2cMasterStart(((deviceAddr<<1) | I2C_WRITE) | eepromA1)) return false;
		if (!SoftI2cMasterWrite((readAddress & 0xFF))) return false;
		if (!SoftI2cMasterRestart(((deviceAddr<<1) | I2C_READ) | eepromA1)) return false;
	}
	else if (readAddress > eeprom8Kbit && readAddress <= eeprom16Kbit) { // Set A2 bit
		if (!SoftI2cMasterStart(((deviceAddr<<1) | I2C_WRITE) | eepromA2)) return false;
		if (!SoftI2cMasterWrite((readAddress & 0xFF))) return false;
		if (!SoftI2cMasterRestart(((deviceAddr<<1) | I2C_READ) | eepromA2)) return false;
	}
	else { // 32Kbit+ EEPROM
		if (!SoftI2cMasterStart((deviceAddr<<1) | I2C_WRITE)) return false;
		if (!SoftI2cMasterWrite((readAddress >> 8))) return false; // MSB
		if (!SoftI2cMasterWrite((readAddress & 0xFF))) return false; // LSB
		if (!SoftI2cMasterRestart((deviceAddr<<1) | I2C_READ)) return false;
	}
	
	// Read the byte
	byteRead = SoftI2cMasterRead(1);
	
	// Issue a stop condition
	SoftI2cMasterStop();
	
	return byteRead;
}

// Write 1 byte to the address without caring about the EEPROM size
// This function should not be used by anything other than eepromSizetest
bool soft_i2c_eeprom_write_minimal(uint8_t deviceAddr, uint16_t writeAddress, byte writeByte) {
	
	if (writeAddress <= eeprom2Kbit) { // 1-2Kbit EEPROM
		if (!SoftI2cMasterStart((deviceAddr<<1) | I2C_WRITE)) return false;
		if (!SoftI2cMasterWrite(writeAddress)) return false;
	}
	else if (writeAddress > eeprom2Kbit && writeAddress <= eeprom4Kbit) { // Set A0 bit
		if (!SoftI2cMasterStart(((deviceAddr<<1) | I2C_WRITE) | eepromA0)) return false;
		if (!SoftI2cMasterWrite((writeAddress & 0xFF))) return false;
	}
	else if (writeAddress > eeprom4Kbit && writeAddress <= eeprom8Kbit) { // Set A1 bit
		if (!SoftI2cMasterStart(((deviceAddr<<1) | I2C_WRITE) | eepromA1)) return false;
		if (!SoftI2cMasterWrite((writeAddress & 0xFF))) return false;
	}
	else if (writeAddress > eeprom8Kbit && writeAddress <= eeprom16Kbit) { // Set A2 bit
		if (!SoftI2cMasterStart(((deviceAddr<<1) | I2C_WRITE) | eepromA2)) return false;
		if (!SoftI2cMasterWrite((writeAddress & 0xFF))) return false;
	}
	else { // 32Kbit+ EEPROM
		if (!SoftI2cMasterStart((deviceAddr<<1) | I2C_WRITE)) return false;
		if (!SoftI2cMasterWrite((writeAddress >> 8))) return false; // MSB
		if (!SoftI2cMasterWrite((writeAddress & 0xFF))) return false; // LSB
	}
	
	// Write the byte
	if (!SoftI2cMasterWrite(writeByte)) return false;

	// Issue a stop condition
	SoftI2cMasterStop();
	
	// Delay 10ms for the write to complete, depends on the EEPROM you use
	// We divide 10ms by 16.5MHz as we are at 1MHz when doing this delay
	_delay_us(650);
	
	return true;
}

// Test what EEPROM which is inserted by writing and reading the last memory location of 1Kbit, 2Kbit, etc 
// with different values, if the values don't match then we know the EEPROM memory limit has been reached
void eepromSizetest(void) {
	
	int i = 9; // Start off checking the highest EEPROM (512Kbit)
	
	// Check if it's an EEPROM with an 8bit address (if we don't check for this and run a write made for a 16bit 
	// addressable chip on an 8bit chip, it will write the second byte of the address to the first byte of the address)
	SoftI2cMasterInit();
	int saveByte = soft_i2c_eeprom_read_minimal(EEPROM_ADDR, eepromMap[0]);
	soft_i2c_eeprom_write_minimal(EEPROM_ADDR, eepromMap[0], 1);
	int readNumber = soft_i2c_eeprom_read_minimal(EEPROM_ADDR, eepromMap[0]);
	soft_i2c_eeprom_write_minimal(EEPROM_ADDR, eepromMap[0], saveByte);
	SoftI2cMasterDeInit();
	
	if (readNumber == 1) { 
		i = 4; // Detected 8bit addressable EEPROM, start off at highest 8bit address EEPROM (16Kbit)
	}
	
	// Start checking for EEPROM size
	while (i > 0) {
		SoftI2cMasterInit();
		
		// Save the data that is at the location we will write to
		int saveByte = soft_i2c_eeprom_read_minimal(EEPROM_ADDR, eepromMap[i]);
		int saveByte2 = soft_i2c_eeprom_read_minimal(EEPROM_ADDR, eepromMap[i-1]);
		
		// Write/read test
		soft_i2c_eeprom_write_minimal(EEPROM_ADDR, eepromMap[i], i);
		soft_i2c_eeprom_write_minimal(EEPROM_ADDR, eepromMap[i-1], i-1);
		int readNumber = soft_i2c_eeprom_read_minimal(EEPROM_ADDR, eepromMap[i]);
		
		// Restore the data that was originally at the location
		soft_i2c_eeprom_write_minimal(EEPROM_ADDR, eepromMap[i], saveByte);
		soft_i2c_eeprom_write_minimal(EEPROM_ADDR, eepromMap[i-1], saveByte2);
		
		// If we can still read the first value that we wrote to a higher location, we found the EEPROM size
		if (readNumber == i) { 
			eepromMemsize = i;
			break;
		}
		
		// When do a read/write test to the 32Kbit, we would actually do test read/write for a 16Kbit EEPROM (i-1) and that 
		// will cause problems because a 16Kbit EEPROM doesn't use 16 bit addresses. So on i == 6, we'll test if 
		// readNumber equals 5 which means we have a 32Kbit EEPROM as that's the lowest 16bit addressable EEPROM available.
		if (i == 6 && readNumber == 5) {
			eepromMemsize = 5;
			break;
		}
		
		SoftI2cMasterDeInit();
		i--;
	}
	
	// Try the smallest EEPROM size by itself if no other EEPROM size was found
	if (eepromMemsize == -1) {
		SoftI2cMasterInit();
		int saveByte = soft_i2c_eeprom_read_minimal(EEPROM_ADDR, eepromMap[0]);
		soft_i2c_eeprom_write_minimal(EEPROM_ADDR, eepromMap[0], 1);
		int readNumber = soft_i2c_eeprom_read_minimal(EEPROM_ADDR, eepromMap[0]);
		soft_i2c_eeprom_write_minimal(EEPROM_ADDR, eepromMap[0], saveByte);
		
		if (readNumber == 1) { 
			SoftI2cMasterDeInit();
			eepromMemsize = 0;
		}
	}
	SoftI2cMasterDeInit();
}

// Modified from http://www.arduino.cc/playground/ComponentLib/Thermistor2
// Learn more at http://www.insidegadgets.com/2011/12/18/verifying-the-thermistor-function-for-temperature-reading/
double thermistorTemp(int RawADC) {
  double Temp;
  Temp = log((double) ((10240000/RawADC) - 10000) / 10000); // We divide by our thermistor's resistance at 25C, in this case 10K
  Temp = 1 / (0.003354016 + (0.0002569850 * Temp) + (0.000002620131 * Temp * Temp) + (0.00000006383091 * Temp * Temp * Temp));
  Temp = Temp - 273.15; // Convert Kelvin to Celsius
  return Temp;
}

// Used from http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void system_sleep(void) {
	cbi(ADCSRA,ADEN); // Switch Analog to Digital converter OFF
	set_sleep_mode(SLEEP_MODE_PWR_DOWN); // Set sleep mode
	sleep_mode(); // System sleeps here
	sbi(ADCSRA,ADEN);  // Switch Analog to Digital converter ON
}

// Watchdog timeout values
// 0=16ms, 1=32ms, 2=64ms, 3=128ms, 4=250ms, 5=500ms, 6=1sec, 7=2sec, 8=4sec, 9=8sec
// Used from http://interface.khm.de/index.php/lab/experiments/sleep_watchdog_battery/
void setup_watchdog(byte ii) {
	byte bb;
	if (ii > 9 ) ii=9;
	bb=ii & 7;
	if (ii > 7) bb|= (1<<5);
	bb|= (1<<WDCE);
	
	MCUSR &= ~(1<<WDRF);
	// Start timed sequence
	WDTCR |= (1<<WDCE) | (1<<WDE);
	// Set new watchdog timeout value
	WDTCR = bb;
	WDTCR |= _BV(WDIE);
}

// Turn off the Watchdog
void turnOffwatchdog(void) {
	cli(); // Turn interrupts off
	MCUSR &= ~(1<<WDRF);
	WDTCR |= (1<<WDCE) | (1<<WDE);
	WDTCR = 0;
	sei(); // Turn interrupts back on
}

// Used from Arduino wiring_analog.c
int analogRead(uint8_t pin) {
	uint8_t low, high;
	
	// set the analog reference (high two bits of ADMUX) and select the
	// channel (low 4 bits).  this also sets ADLAR (left-adjust result)
	// to 0 (the default).
	// ADMUX = (analog_reference << 6) | (pin & 0x3f); // more MUX
	// sapo per tiny45
	if (functionInuse == functionTemperature || functionInuse == function2Bytetemperature) {
		ADMUX = pin & 0x3f;
	}
	else {
		ADMUX = (1 << 7) | (pin & 0x3f); // 1.1V voltage reference
	}
	
	// without a delay, we seem to read from the wrong channel
	//delay(1);
	
	// start the conversion
	//sbi(ADCSRA, ADSC);
	
	// ADSC is cleared when the conversion finishes
	//while (bit_is_set(ADCSRA, ADSC));
	
	set_sleep_mode(SLEEP_MODE_ADC); // Set sleep mode ADC
	sleep_mode(); // System sleeps here
	
	// we have to read ADCL first; doing so locks both ADCL
	// and ADCH until ADCH is read.  reading ADCL second would
	// cause the results of each conversion to be discarded,
	// as ADCL and ADCH would be locked when it completed.
	low = ADCL;
	high = ADCH;
	
	// combine the two bytes
	return (high << 8) | low;
}

// Used and modified from Arduino wiring.c - to setup the ATtiny
void setup(void) {
	
	// Change 2MHz to 1 MHz by changing clock prescaler to 16
	CLKPR = (1<<CLKPCE); // Prescaler enable
	CLKPR = (1<<CLKPS2); // Clock division factor 16 (0100)
	
	sei(); // Turn on interrupts
	
	// Set ADC prescale factor to 8
	// 1 MHz / 8 = 125 KHz, inside the desired 50-200 KHz range.
	sbi(ADCSRA, ADPS1);
	sbi(ADCSRA, ADPS0);
	
	// Enable ADC conversions
	sbi(ADCSRA, ADEN);
	
	// Turn on ADC interrupt
	sbi(ADCSRA, ADIE);
}

// Begin V-USB default required code
/* ------------------------------------------------------------------------- */
/* ------------------------ Oscillator Calibration ------------------------- */
/* ------------------------------------------------------------------------- */

/* Calibrate the RC oscillator to 8.25 MHz. The core clock of 16.5 MHz is
 * derived from the 66 MHz peripheral clock by dividing. Our timing reference
 * is the Start Of Frame signal (a single SE0 bit) available immediately after
 * a USB RESET. We first do a binary search for the OSCCAL value and then
 * optimize this value with a neighboorhod search.
 * This algorithm may also be used to calibrate the RC oscillator directly to
 * 12 MHz (no PLL involved, can therefore be used on almost ALL AVRs), but this
 * is wide outside the spec for the OSCCAL value and the required precision for
 * the 12 MHz clock! Use the RC oscillator calibrated to 12 MHz for
 * experimental purposes only!
 */
static void calibrateOscillator(void)
{
uchar       step = 128;
uchar       trialValue = 0, optimumValue;
int         x, optimumDev, targetValue = (unsigned)(1499 * (double)F_CPU / 10.5e6 + 0.5);

    /* do a binary search: */
    do{
        OSCCAL = trialValue + step;
        x = usbMeasureFrameLength();    /* proportional to current real frequency */
        if(x < targetValue)             /* frequency still too low */
            trialValue += step;
        step >>= 1;
    }while(step > 0);
    /* We have a precision of +/- 1 for optimum OSCCAL here */
    /* now do a neighborhood search for optimum value */
    optimumValue = trialValue;
    optimumDev = x; /* this is certainly far away from optimum */
    for(OSCCAL = trialValue - 1; OSCCAL <= trialValue + 1; OSCCAL++){
        x = usbMeasureFrameLength() - targetValue;
        if(x < 0)
            x = -x;
        if(x < optimumDev){
            optimumDev = x;
            optimumValue = OSCCAL;
        }
    }
    OSCCAL = optimumValue;
}
/*
Note: This calibration algorithm may try OSCCAL values of up to 192 even if
the optimum value is far below 192. It may therefore exceed the allowed clock
frequency of the CPU in low voltage designs!
You may replace this search algorithm with any other algorithm you like if
you have additional constraints such as a maximum CPU clock.
For version 5.x RC oscillators (those with a split range of 2x128 steps, e.g.
ATTiny25, ATTiny45, ATTiny85), it may be useful to search for the optimum in
both regions.
*/

void usbEventResetReady(void)
{
	calibrateOscillator();
}
// End V-USB default required code

// Start USB communications
void startUSB(void) {
	usbDeviceDisconnect();
	_delay_ms(300); // 300 ms disconnect
	usbDeviceConnect();
	usbInit();
}
